@@ -14,13 +14,24 @@ function-to-be-minimized (residual function) in terms of these Parameters.
 from copy import deepcopy
 import numpy as np
 from numpy import (dot, eye, ndarray, ones_like,
-                   sqrt, take, transpose, triu, deprecate)
+                   sqrt, take, transpose, triu)
 from numpy.dual import inv
 from numpy.linalg import LinAlgError
 import multiprocessing
 import numbers
 
+##
+##  scipy version notes:
+##  currently scipy 0.14 is required.
+##  feature           scipy version added
+##    minimize              0.11
+##    OptimizeResult        0.13
+##    diff_evolution        0.15
+##    least_squares         0.17
+##
+
 from scipy.optimize import leastsq as scipy_leastsq
+from scipy.optimize import minimize as scipy_minimize
 
 # differential_evolution is only present in scipy >= 0.15
 try:
@@ -28,6 +39,14 @@ try:
 except ImportError:
     from ._differentialevolution import differential_evolution as scipy_diffev
 
+# check for scipy.opitimize.least_squares
+HAS_LEAST_SQUARES = False
+try:
+    from scipy.optimize import least_squares
+    HAS_LEAST_SQUARES = True
+except ImportError:
+    pass
+
 # check for EMCEE
 HAS_EMCEE = False
 try:
@@ -44,22 +63,6 @@ try:
 except ImportError:
     pass
 
-# check for scipy.optimize.minimize
-HAS_SCALAR_MIN = False
-try:
-    from scipy.optimize import minimize as scipy_minimize
-    HAS_SCALAR_MIN = True
-except ImportError:
-    pass
-
-# check for scipy.opitimize.least_squares
-HAS_LEAST_SQUARES = False
-try:
-    from scipy.optimize import least_squares
-    HAS_LEAST_SQUARES = True
-except ImportError:
-    pass
-
 from .parameter import Parameter, Parameters
 
 # use locally modified version of uncertainties package
@@ -146,24 +149,65 @@ SCALAR_METHODS = {'nelder': 'Nelder-Mead',
 
 
 class MinimizerResult(object):
-    """ The result of a minimization.
+    """
+    A class that holds the results of a minimization.
+
+    This is a plain container (with no methods of its own) that
+    simply holds the results of the minimization.  Fit results
+    include data such as status and error messages, fit statistics,
+    and the updated (i.e. best-fit) parameters themselves :attr:`params`.
+
+    The list of (possible) `MinimizerResult` attributes follows.
 
     Attributes
     ----------
-    params : Parameters
-        The best-fit parameters
-    success : bool
-        Whether the minimization was successful
+    params : :class:`lmfit.parameters.Parameters`
+        The best-fit parameters resulting from the fit.
     status : int
         Termination status of the optimizer. Its value depends on the
         underlying solver. Refer to `message` for details.
+    var_names : list
+        Ordered list of variable parameter names used in optimization, and
+        useful for understanding the the values in :attr:`init_vals` and
+        :attr:`covar`.
+    covar : numpy.ndarray
+        covariance matrix from minimization (`leastsq` only), with
+        rows/columns using :attr:`var_names`.
+    init_vals : list
+        List of initial values for variable parameters using :attr:`var_names`.
+    init_values : dict
+        Dictionary of initial values for variable parameters.
+    nfev : int
+        Number of function evaluations
+    success : bool
+        Boolean (``True``/``False``) for whether fit succeeded.
+    errorbars : bool
+        Boolean (``True``/``False``) for whether uncertainties were estimated.
+    message : string
+        Message about fit success.
+    ier : int
+        Integer error value from :scipydoc:`optimize.leastsq` (`leastsq` only).
+    lmdif_message : string
+        Message from :scipydoc:`optimize.leastsq` (`leastsq` only).
+    nvarys : int
+        Number of variables in fit :math:`N_{\\rm varys}`.
+    ndata : int
+        Number of data points:  :math:`N`
+    nfree : int
+        Degrees of freedom in fit:  :math:`N - N_{\\rm varys}`
+    residual : numpy.ndarray
+        Residual array :math:`{\\rm Resid_i}`. Return value of the objective
+        function.
+    chisqr : float
+        Chi-square: :math:`\chi^2 = \sum_i^N [{\\rm Resid}_i]^2`
+    redchi : float
+        Reduced chi-square:
+        :math:`\chi^2_{\\nu}= {\chi^2} / {(N - N_{\\rm varys})}`
+    aic : float
+        Akaike Information Criterion statistic.
+    bic : float
+        Bayesian Information Criterion statistic.
 
-    Notes
-    -----
-    Additional attributes not listed above may be present, depending on the
-    specific solver. Since this class is essentially a subclass of dict
-    with attribute accessors, one can see which attributes are available
-    using the `keys()` method.
     """
     def __init__(self, **kws):
         for key, val in kws.items():
@@ -187,11 +231,11 @@ class MinimizerResult(object):
 
 class Minimizer(object):
     """A general minimizer for curve fitting"""
-    err_nonparam = ("params must be a minimizer.Parameters() instance or list "
-                    "of Parameters()")
-    err_maxfev = ("Too many function calls (max set to %i)!  Use:"
-                  " minimize(func, params, ..., maxfev=NNN)"
-                  "or set leastsq_kws['maxfev']  to increase this maximum.")
+    _err_nonparam = ("params must be a minimizer.Parameters() instance or list "
+                     "of Parameters()")
+    _err_maxfev = ("Too many function calls (max set to %i)!  Use:"
+                   " minimize(func, params, ..., maxfev=NNN)"
+                   "or set leastsq_kws['maxfev']  to increase this maximum.")
 
     def __init__(self, userfcn, params, fcn_args=None, fcn_kws=None,
                  iter_cb=None, scale_covar=True, nan_policy='raise',
@@ -206,12 +250,12 @@ class Minimizer(object):
             model and data) to be minimized in a least squares sense.  The
             function must have the signature:
             `userfcn(params, *fcn_args, **fcn_kws)`
-        params : lmfit.parameter.Parameters object.
+        params : :class:`lmfit.parameter.Parameters` object.
             contains the Parameters for the model.
         fcn_args : tuple, optional
-            positional arguments to pass to userfcn.
+            positional arguments to pass to `userfcn`.
         fcn_kws : dict, optional
-            keyword arguments to pass to userfcn.
+            keyword arguments to pass to `userfcn`.
         iter_cb : callable, optional
             Function to be called at each fit iteration. This function should
             have the signature:
@@ -236,7 +280,8 @@ class Minimizer(object):
         Notes
         -----
         The objective function should return the value to be minimized. For the
-        Levenberg-Marquardt algorithm from leastsq(), this returned value must
+        Levenberg-Marquardt algorithm from :meth:`leastsq` or
+        :meth:`least_squares`, this returned value must
         be an array, with a length greater than or equal to the number of
         fitting variables in the model. For the other methods, the return value
         can either be a scalar or an array. If an array is returned, the sum of
@@ -245,10 +290,10 @@ class Minimizer(object):
         the objective function returns non-finite values then a `ValueError`
         will be raised because the underlying solvers cannot deal with them.
 
-        A common use for the fcn_args and fcn_kwds would be to pass in other
-        data needed to calculate the residual, including such things as the
-        data array, dependent variable, uncertainties in the data, and other
-        data structures for the model calculation.
+        A common use for the `fcn_args` and `fcn_kwds` would be to pass in
+        other data needed to calculate the residual, including such things
+        as the data array, dependent variable, uncertainties in the data,
+        and other data structures for the model calculation.
         """
         self.userfcn = userfcn
         self.userargs = fcn_args
@@ -281,14 +326,9 @@ class Minimizer(object):
 
     @property
     def values(self):
+        """dict : Parameter values in a simple dictionary.
         """
-        Returns
-        -------
-        param_values : dict
-            Parameter values in a simple dictionary.
-        """
-
-        return dict([(name, p.value) for name, p in self.result.params.items()])
+        return {name: p.value for name, p in self.result.params.items()}
 
     def __residual(self, fvars, apply_bounds_transformation=True):
         """
@@ -369,15 +409,16 @@ class Minimizer(object):
 
     def penalty(self, fvars):
         """
-        Penalty function for scalar minimizers:
+        Penalty function for scalar minimizers.
 
         Parameters
         ----------
-        fvars : array of values for the variable parameters
+        fvars : numpy.ndarray
+            Array of values for the variable parameters
 
         Returns
         -------
-        r - float
+        r : float
             The user evaluated user-supplied objective function. If the
             objective function is an array, return the array sum-of-squares
         """
@@ -388,8 +429,7 @@ class Minimizer(object):
 
     def prepare_fit(self, params=None):
         """
-        Prepares parameters for fitting,
-        return array of initial values
+        Prepares parameters for fitting, return array of initial values.
         """
         # determine which parameters are actually variables
         # and which are defined expressions.
@@ -403,11 +443,11 @@ class Minimizer(object):
             result.params = Parameters()
             for par in self.params:
                 if not isinstance(par, Parameter):
-                    raise MinimizerException(self.err_nonparam)
+                    raise MinimizerException(self._err_nonparam)
                 else:
                     result.params[par.name] = par
         elif self.params is None:
-            raise MinimizerException(self.err_nonparam)
+            raise MinimizerException(self._err_nonparam)
 
         # determine which parameters are actually variables
         # and which are defined expressions.
@@ -431,21 +471,23 @@ class Minimizer(object):
             if par.name is None:
                 par.name = name
         result.nvarys = len(result.var_names)
+        result.init_values = {n: v for n, v in zip(result.var_names,
+                                                   result.init_vals)}
         return result
 
     def unprepare_fit(self):
         """
-        Unprepares the fit, so that subsequent fits will be
-        forced to run prepare_fit.
+        Clean fit state, so that subsequent fits will need to call prepare_fit.
 
-        removes ast compilations of constraint expressions
+        Removes AST compilations of constraint expressions.
         """
         pass
 
     def scalar_minimize(self, method='Nelder-Mead', params=None, **kws):
         """
-        Use one of the scalar minimization methods from
-        scipy.optimize.minimize.
+        Scalar minimization using `scipy.optimize.minimize`.
+
+
 
         Parameters
         ----------
@@ -467,7 +509,7 @@ class Minimizer(object):
         params : Parameters, optional
            Parameters to use as starting points.
         kws : dict, optional
-            Minimizer options pass to scipy.optimize.minimize.
+            Minimizer options pass to `scipy.optimize.minimize`.
 
         If the objective function returns a numpy array instead
         of the expected scalar, the sum of squares of the array
@@ -481,14 +523,17 @@ class Minimizer(object):
 
         Returns
         -------
-        success : bool
-            Whether the fit was successful.
+        :class:`MinimizerResult`
+            Object containing the optimized parameter
+            and several goodness-of-fit statistics.
 
+
+        .. versionchanged:: 0.9.0
+           return value changed to :class:`MinimizerResult`
         """
-        if not HAS_SCALAR_MIN:
-            raise NotImplementedError
 
         result = self.prepare_fit(params=params)
+        result.method = method
         vars = result.init_vals
         params = result.params
 
@@ -562,6 +607,8 @@ class Minimizer(object):
               ntemps=1, pos=None, reuse_sampler=False, workers=1,
               float_behavior='posterior', is_weighted=True, seed=None):
         """
+        Bayesian sampling of the posterior distribution using the `emcee`.
+
         Bayesian sampling of the posterior distribution for the parameters
         using the `emcee` Markov Chain Monte Carlo package. The method assumes
         that the prior is Uniform. You need to have `emcee` installed to use
@@ -653,7 +700,7 @@ class Minimizer(object):
 
         Returns
         -------
-        result : MinimizerResult
+        :class:`MinimizerResult`
             MinimizerResult object containing updated params, statistics,
             etc. The `MinimizerResult` also contains the ``chain``,
             ``flatchain`` and ``lnprob`` attributes. The ``chain``
@@ -749,6 +796,7 @@ class Minimizer(object):
             tparams = None
 
         result = self.prepare_fit(params=tparams)
+        result.method = 'emcee'
         params = result.params
 
         # check if the userfcn returns a vector of residuals
@@ -910,11 +958,11 @@ class Minimizer(object):
     def least_squares(self, params=None, **kws):
         """
         Use the least_squares (new in scipy 0.17) function to perform a fit.
-        This assumes that Parameters have been stored, and a function to
-        minimize has been properly set up.
 
-        This wraps scipy.optimize.least_squares, which has inbuilt support
-        for bounds and robust loss functions.
+        This method assumes that Parameters have been stored, and a function to
+        minimize has been properly set up.
+        This method wraps scipy.optimize.least_squares, which has inbuilt
+        support for bounds and robust loss functions.
 
         Parameters
         ----------
@@ -922,6 +970,16 @@ class Minimizer(object):
            Parameters to use as starting points.
         kws : dict, optional
             Minimizer options to pass to scipy.optimize.least_squares.
+
+        Returns
+        -------
+        :class:`MinimizerResult`
+            Object containing the optimized parameter
+            and several goodness-of-fit statistics.
+
+
+        .. versionchanged:: 0.9.0
+           return value changed to :class:`MinimizerResult`
         """
 
         if not HAS_LEAST_SQUARES:
@@ -929,6 +987,7 @@ class Minimizer(object):
                                       "is needed for this method.")
 
         result = self.prepare_fit(params)
+        result.method = 'least_squares'
 
         replace_none = lambda x, sign: sign*np.inf if x is None else x
         upper_bounds = [replace_none(i.max, 1) for i in self.params.values()]
@@ -983,10 +1042,15 @@ class Minimizer(object):
 
         Returns
         -------
-        success : bool
-            True if fit was successful, False if not.
+        :class:`MinimizerResult`
+            Object containing the optimized parameter
+            and several goodness-of-fit statistics.
+
+        .. versionchanged:: 0.9.0
+           return value changed to :class:`MinimizerResult`
         """
         result = self.prepare_fit(params=params)
+        result.method = 'leastsq'
         vars = result.init_vals
         nvars = len(vars)
         lskws = dict(full_output=1, xtol=1.e-7, ftol=1.e-7, col_deriv=False,
@@ -1021,7 +1085,7 @@ class Minimizer(object):
         elif ier == 0:
             result.message = 'Invalid Input Parameters.'
         elif ier == 5:
-            result.message = self.err_maxfev % lskws['maxfev']
+            result.message = self._err_maxfev % lskws['maxfev']
         else:
             result.message = 'Tolerance seems to be too small.'
 
@@ -1121,30 +1185,43 @@ class Minimizer(object):
         Parameters
         ----------
         method : str, optional
-            Name of the fitting method to use.
-            One of:
-            'leastsq'                -    Levenberg-Marquardt (default)
-            'nelder'                 -    Nelder-Mead
-            'lbfgsb'                 -    L-BFGS-B
-            'powell'                 -    Powell
-            'cg'                     -    Conjugate-Gradient
-            'newton'                 -    Newton-CG
-            'cobyla'                 -    Cobyla
-            'tnc'                    -    Truncate Newton
-            'trust-ncg'              -    Trust Newton-CGn
-            'dogleg'                 -    Dogleg
-            'slsqp'                  -    Sequential Linear Squares Programming
-            'differential_evolution' -    differential evolution
+            Name of the fitting method to use. Valid values are:
 
-        params : Parameters, optional
-            parameters to use as starting values
+            - `'leastsq'`: Levenberg-Marquardt (default).
+              Uses `scipy.optimize.leastsq`.
+            - `'least_squares'`: Levenberg-Marquardt.
+              Uses `scipy.optimize.least_squares`.
+            - 'nelder': Nelder-Mead
+            - `'lbfgsb'`: L-BFGS-B
+            - `'powell'`: Powell
+            - `'cg'`: Conjugate-Gradient
+            - `'newton'`: Newton-CG
+            - `'cobyla'`: Cobyla
+            - `'tnc'`: Truncate Newton
+            - `'trust-ncg'`: Trust Newton-CGn
+            - `'dogleg'`: Dogleg
+            - `'slsqp'`: Sequential Linear Squares Programming
+            - `'differential_evolution'`: differential evolution
+
+            For more details on the fitting methods please refer to the
+            `scipy docs <http://docs.scipy.org/doc/scipy/reference/optimize.html>`__.
+
+        params : :class:`lmfit.parameter.Parameters` object.
+            Parameters of the model to use as starting values.
+
+        **kwargs
+            Additional arguments are passed to the underlying minimization
+            method.
 
         Returns
         -------
-        result : MinimizerResult
+        :class:`MinimizerResult`
+            Object containing the optimized parameter
+            and several goodness-of-fit statistics.
 
-            MinimizerResult object contains updated params, fit statistics, etc.
 
+        .. versionchanged:: 0.9.0
+           return value changed to :class:`MinimizerResult`
         """
 
         function = self.leastsq
@@ -1152,19 +1229,16 @@ class Minimizer(object):
         kwargs.update(kws)
 
         user_method = method.lower()
-        if user_method.startswith('least'):
+        if user_method.startswith('leasts'):
             function = self.leastsq
-        elif HAS_SCALAR_MIN:
+        elif user_method.startswith('least_s'):
+            function = self.least_squares
+        else:
             function = self.scalar_minimize
             for key, val in SCALAR_METHODS.items():
                 if (key.lower().startswith(user_method) or
                     val.lower().startswith(user_method)):
                     kwargs['method'] = val
-        elif (user_method.startswith('nelder') or
-              user_method.startswith('fmin')):
-            function = self.fmin
-        elif user_method.startswith('lbfgsb'):
-            function = self.lbfgsb
         return function(**kwargs)
 
 
@@ -1323,8 +1397,9 @@ def _nan_policy(a, nan_policy='raise', handle_inf=True):
     Returns
     -------
     filtered_array : array_like
-    """
-    """
+
+    Note
+    ----
     This function is copied, then modified, from
     scipy/stats/stats.py/_contains_nan
     """
@@ -1369,40 +1444,50 @@ def _nan_policy(a, nan_policy='raise', handle_inf=True):
 def minimize(fcn, params, method='leastsq', args=None, kws=None,
              scale_covar=True, iter_cb=None, **fit_kws):
     """
-    A general purpose curvefitting function
-    The minimize function takes a objective function to be minimized, a
-    dictionary (lmfit.parameter.Parameters) containing the model parameters,
-    and several optional arguments.
+    This function performs a fit of a set of parameters by minimizing
+    an objective (or "cost") function using one one of the several
+    available methods. The minimize function takes a objective function
+    to be minimized, a dictionary (:class:`lmfit.parameter.Parameters`)
+    containing the model parameters, and several optional arguments.
 
     Parameters
     ----------
     fcn : callable
-        objective function that returns the residual (difference between
-        model and data) to be minimized in a least squares sense.  The
-        function must have the signature:
+        objective function to be minimized. When method is `leastsq` or
+        `least_squares`, the objective function should return an array
+        of residuals (difference between model and data) to be minimized
+        in a least squares sense. With the scalar methods the objective
+        function can either return the residuals array or a single scalar
+        value. The function must have the signature:
         `fcn(params, *args, **kws)`
-    params : lmfit.parameter.Parameters object.
+    params : :class:`lmfit.parameter.Parameters` object.
         contains the Parameters for the model.
     method : str, optional
-        Name of the fitting method to use.
-        One of:
-            'leastsq'                -    Levenberg-Marquardt (default)
-            'nelder'                 -    Nelder-Mead
-            'lbfgsb'                 -    L-BFGS-B
-            'powell'                 -    Powell
-            'cg'                     -    Conjugate-Gradient
-            'newton'                 -    Newton-CG
-            'cobyla'                 -    Cobyla
-            'tnc'                    -    Truncate Newton
-            'trust-ncg'              -    Trust Newton-CGn
-            'dogleg'                 -    Dogleg
-            'slsqp'                  -    Sequential Linear Squares Programming
-            'differential_evolution' -    differential evolution
+        Name of the fitting method to use. Valid values are:
+
+        - `'leastsq'`: Levenberg-Marquardt (default).
+          Uses `scipy.optimize.leastsq`.
+        - `'least_squares'`: Levenberg-Marquardt.
+          Uses `scipy.optimize.least_squares`.
+        - 'nelder': Nelder-Mead
+        - `'lbfgsb'`: L-BFGS-B
+        - `'powell'`: Powell
+        - `'cg'`: Conjugate-Gradient
+        - `'newton'`: Newton-CG
+        - `'cobyla'`: Cobyla
+        - `'tnc'`: Truncate Newton
+        - `'trust-ncg'`: Trust Newton-CGn
+        - `'dogleg'`: Dogleg
+        - `'slsqp'`: Sequential Linear Squares Programming
+        - `'differential_evolution'`: differential evolution
+
+        For more details on the fitting methods please refer to the
+        `scipy docs <http://docs.scipy.org/doc/scipy/reference/optimize.html>`__.
 
     args : tuple, optional
-        Positional arguments to pass to fcn.
+        Positional arguments to pass to `fcn`.
     kws : dict, optional
-        keyword arguments to pass to fcn.
+        keyword arguments to pass to `fcn`.
     iter_cb : callable, optional
         Function to be called at each fit iteration. This function should
         have the signature `iter_cb(params, iter, resid, *args, **kws)`,
@@ -1415,6 +1500,16 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
     fit_kws : dict, optional
         Options to pass to the minimizer being used.
 
+    Returns
+    -------
+    :class:`MinimizerResult`
+        Object containing the optimized parameter
+        and several goodness-of-fit statistics.
+
+
+    .. versionchanged:: 0.9.0
+        return value changed to :class:`MinimizerResult`.
+
     Notes
     -----
     The objective function should return the value to be minimized. For the
@@ -1429,6 +1524,19 @@ def minimize(fcn, params, method='leastsq', args=None, kws=None,
     data needed to calculate the residual, including such things as the
     data array, dependent variable, uncertainties in the data, and other
     data structures for the model calculation.
+
+    On output, the params will be unchanged.  The best-fit values, and where
+    appropriate, estimated uncertainties and correlations, will all be
+    contained in the returned :class:`MinimizerResult`.  See
+    :ref:`fit-results-label` for further details.
+
+    This function is simply a wrapper around :class:`Minimizer`
+    and is equivalent to::
+
+        fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
+        	           iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
+        fitter.minimize(method=method)
+
     """
     fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
                        iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
