@@ -8,14 +8,14 @@ import operator
 from copy import deepcopy
 import numpy as np
 from . import Parameters, Parameter, Minimizer
-from .printfuncs import fit_report
+from .printfuncs import fit_report, ci_report
+from .confidence import conf_interval
 
 try:
     from collections import OrderedDict
 except ImportError:
     from ordereddict import OrderedDict
 
-
 # Use pandas.isnull for aligning missing data is pandas is available.
 # otherwise use numpy.isnan
 try:
@@ -105,8 +105,9 @@ class Model(object):
             raise ValueError(self._invalid_missing)
         self.missing = missing
         self.opts = kws
-        self.param_hints = {}
-        self._param_names = set()
+        self.param_hints = OrderedDict()
+        # the following has been changed from OrderedSet for the time being
+        self._param_names = []
         self._parse_params()
         if self.independent_vars is None:
             self.independent_vars = []
@@ -138,30 +139,16 @@ class Model(object):
     def prefix(self):
         return self._prefix
 
-    @prefix.setter
-    def prefix(self, value):
-        self._prefix = value
-        self._parse_params()
-
     @property
     def param_names(self):
         return self._param_names
 
     def __repr__(self):
-        return  "<lmfit.Model: %s>" % (self.name)
+        return "<lmfit.Model: %s>" % (self.name)
 
-    def copy(self, prefix=None):
-        """Return a completely independent copy of the whole model.
-
-        Parameters
-        ----------
-        prefix: string or None. If not None new model's prefix is
-            changed to the passed value.
-        """
-        new = deepcopy(self)
-        if prefix is not None:
-            new.prefix = prefix
-        return new
+    def copy(self, **kwargs):
+        """DOES NOT WORK"""
+        raise NotImplementedError("Model.copy does not work. Make a new Model")
 
     def _parse_params(self):
         "build params from function arguments"
@@ -174,7 +161,7 @@ class Model(object):
         if argspec.defaults is not None:
             for val in reversed(argspec.defaults):
                 kw_args[pos_args.pop()] = val
-        #
+
         self._func_haskeywords = keywords is not None
         self._func_allargs = pos_args + list(kw_args.keys())
         allargs = self._func_allargs
@@ -229,8 +216,8 @@ class Model(object):
             if (self._strip_prefix(arg) not in allargs or
                 arg in self._forbidden_args):
                 raise ValueError(self._invalid_par % (arg, fname))
-
-        self._param_names = set(names)
+        # the following as been changed from OrderedSet for the time being.
+        self._param_names = names[:]
 
     def set_param_hint(self, name, **kwargs):
         """set hints for parameter, including optional bounds
@@ -247,34 +234,58 @@ class Model(object):
             name = name[npref:]
 
         if name not in self.param_hints:
-            self.param_hints[name] = {}
-        hints = self.param_hints[name]
+            self.param_hints[name] = OrderedDict()
+
         for key, val in kwargs.items():
             if key in self._hint_names:
-                hints[key] = val
+                self.param_hints[name][key] = val
             else:
                 warnings.warn(self._invalid_hint % (key, name))
 
-    def make_params(self, **kwargs):
+    def print_param_hints(self, colwidth=8):
+        """Prints a nicely aligned text-table of parameters hints.
+
+        The argument `colwidth` is the width of each column,
+        except for first and last columns.
+        """
+        name_len = max(len(s) for s in self.param_hints)
+        print('{:{name_len}}  {:>{n}} {:>{n}} {:>{n}} {:>{n}}    {:{n}}'
+              .format('Name', 'Value', 'Min', 'Max', 'Vary', 'Expr',
+                      name_len=name_len, n=colwidth))
+        line = ('{name:<{name_len}}  {value:{n}g} {min:{n}g} {max:{n}g} '
+                '{vary!s:>{n}}    {expr}')
+        for name, values in sorted(self.param_hints.items()):
+            pvalues = dict(name=name, value=np.nan, min=-np.inf, max=np.inf,
+                           vary=True, expr='')
+            pvalues.update(**values)
+            print(line.format(name_len=name_len, n=colwidth, **pvalues))
+
+    def make_params(self, verbose=False, **kwargs):
         """create and return a Parameters object for a Model.
         This applies any default values
         """
-        verbose = False
-        if 'verbose' in kwargs:
-            verbose = kwargs['verbose']
         params = Parameters()
 
+        # make sure that all named parameters are in params
         for name in self.param_names:
-            par = Parameter(name=name)
+            if name in params:
+                par = params[name]
+            else:
+                par = Parameter(name=name)
+            par._delay_asteval = True
             basename = name[len(self._prefix):]
             # apply defaults from model function definition
             if basename in self.def_vals:
                 par.value = self.def_vals[basename]
+            if par.value in (None, -np.inf, np.inf, np.nan):
+                for key, val in self.def_vals.items():
+                    if key in name.lower():
+                        par.value = val
             # apply defaults from parameter hints
             if basename in self.param_hints:
                 hint = self.param_hints[basename]
                 for item in self._hint_names:
-                    if item in  hint:
+                    if item in hint:
                         setattr(par, item, hint[item])
             # apply values passed in through kw args
             if basename in kwargs:
@@ -283,21 +294,32 @@ class Model(object):
             if name in kwargs:
                 # kw parameter names with prefix
                 par.value = kwargs[name]
-            params[name] = par
+            params.add(par)
+            if verbose:
+                print( ' - Adding parameter "%s"' % name)
 
-        # add any additional parameters defined in param_hints
+        # next build parameters defined in param_hints
         # note that composites may define their own additional
         # convenience parameters here
         for basename, hint in self.param_hints.items():
             name = "%s%s" % (self._prefix, basename)
-            if name not in params:
-                par = params[name] = Parameter(name=name)
-                for item in self._hint_names:
-                    if item in  hint:
-                        setattr(par, item, hint[item])
-                # Add the new parameter to the self.param_names
-                self._param_names.add(name)
-                if verbose: print( ' - Adding parameter "%s"' % name)
+            if name in params:
+                par = params[name]
+            else:
+                par = Parameter(name=name)
+                params.add(par)
+                if verbose:
+                    print( ' - Adding parameter for hint "%s"' % name)
+            par._delay_asteval = True
+            for item in self._hint_names:
+                if item in  hint:
+                    setattr(par, item, hint[item])
+            # Add the new parameter to self._param_names
+            if name not in self._param_names:
+                self._param_names.append(name)
+
+        for p in params.values():
+            p._delay_asteval = False
         return params
 
     def guess(self, data=None, **kws):
@@ -310,11 +332,33 @@ class Model(object):
         raise NotImplementedError(msg)
 
     def _residual(self, params, data, weights, **kwargs):
-        "default residual:  (data-model)*weights"
+        """default residual:  (data-model)*weights
+
+        If the model returns complex values, the residual is computed by treating the real and imaginary
+        parts separately. In this case, if the weights provided are real, they are assumed to apply equally to the
+        real and imaginary parts. If the weights are complex, the real part of the weights are applied to the real
+        part of the residual and the imaginary part is treated correspondingly.
+
+        Since the underlying scipy.optimize routines expect np.float arrays, the only complex type supported is
+        np.complex.
+
+        The "ravels" throughout are necessary to support pandas.Series.
+        """
         diff = self.eval(params, **kwargs) - data
+
+        if diff.dtype == np.complex:
+            # data/model are complex
+            diff = diff.ravel().view(np.float)
+            if weights is not None:
+                if weights.dtype == np.complex:
+                    # weights are complex
+                    weights = weights.ravel().view(np.float)
+                else:
+                    # real weights but complex data
+                    weights = (weights + 1j * weights).ravel().view(np.float)
         if weights is not None:
             diff *= weights
-        return np.asarray(diff)  # for compatibility with pandas.Series
+        return np.asarray(diff).ravel()  # for compatibility with pandas.Series
 
     def _handle_missing(self, data):
         "handle missing data"
@@ -365,12 +409,7 @@ class Model(object):
 
     def eval(self, params=None, **kwargs):
         """evaluate the model with the supplied parameters"""
-        result = self.func(**self.make_funcargs(params, kwargs))
-        # Handle special case of constant result and one
-        # independent variable (of any dimension).
-        if np.ndim(result) == 0 and len(self.independent_vars) == 1:
-            result = np.tile(result, kwargs[self.independent_vars[0]].shape)
-        return result
+        return self.func(**self.make_funcargs(params, kwargs))
 
     @property
     def components(self):
@@ -388,7 +427,7 @@ class Model(object):
         return {key: self.eval(params=params, **kwargs)}
 
     def fit(self, data, params=None, weights=None, method='leastsq',
-            iter_cb=None, scale_covar=True, verbose=True, fit_kws=None, **kwargs):
+            iter_cb=None, scale_covar=True, verbose=False, fit_kws=None, **kwargs):
         """Fit the model to the data.
 
         Parameters
@@ -409,7 +448,7 @@ class Model(object):
 
         Returns
         -------
-        lmfit.ModelFit
+        lmfit.ModelResult
 
         Examples
         --------
@@ -437,7 +476,7 @@ class Model(object):
             params = deepcopy(params)
 
         # If any kwargs match parameter names, override params.
-        param_kwargs = set(kwargs.keys()) & self.param_names
+        param_kwargs = set(kwargs.keys()) & set(self.param_names)
         for name in param_kwargs:
             p = kwargs[name]
             if isinstance(p, Parameter):
@@ -449,7 +488,7 @@ class Model(object):
 
         # All remaining kwargs should correspond to independent variables.
         for name in kwargs.keys():
-            if not name in self.independent_vars:
+            if name not in self.independent_vars:
                 warnings.warn("The keyword argument %s does not" % name +
                               "match any arguments of the model function." +
                               "It will be ignored.", UserWarning)
@@ -496,8 +535,9 @@ class Model(object):
         if fit_kws is None:
             fit_kws = {}
 
-        output = ModelFit(self, params, method=method, iter_cb=iter_cb,
-                          scale_covar=scale_covar, fcn_kws=kwargs, **fit_kws)
+        output = ModelResult(self, params, method=method, iter_cb=iter_cb,
+                             scale_covar=scale_covar, fcn_kws=kwargs,
+                             **fit_kws)
         output.fit(data=data, weights=weights)
         output.components = self.components
         return output
@@ -562,7 +602,7 @@ class CompositeModel(Model):
         self.right = right
         self.op    = op
 
-        name_collisions = left.param_names & right.param_names
+        name_collisions = set(left.param_names) & set(right.param_names)
         if len(name_collisions) > 0:
             msg = ''
             for collision in name_collisions:
@@ -610,7 +650,7 @@ class CompositeModel(Model):
 
     @property
     def param_names(self):
-        return self.left.param_names | self.right.param_names
+        return  self.left.param_names + self.right.param_names
 
     @property
     def components(self):
@@ -623,7 +663,8 @@ class CompositeModel(Model):
         out.update(self.left._make_all_args(params=params, **kwargs))
         return out
 
-class ModelFit(Minimizer):
+
+class ModelResult(Minimizer):
     """Result from Model fit
 
     Attributes
@@ -648,20 +689,20 @@ class ModelFit(Minimizer):
          the previous fit.  This allows easily changing data or
          parameter settings, or both.
 
-    eval(**kwargs)
-         evaluate the current model, with the current parameter values,
-         with values in kwargs sent to the model function.
+    eval(params=None, **kwargs)
+         evaluate the current model, with parameters (defaults to the current
+         parameter values), with values in kwargs sent to the model function.
 
-    eval_components(**kwargs)
-         evaluate the current model, with the current parameter values,
-         with values in kwargs sent to the model function and returns
-         a ordered dict with the model names as the key and the component
-         results as the values.
+    eval_components(params=Nones, **kwargs)
+         evaluate the current model, with parameters (defaults to the current
+         parameter values), with values in kwargs sent to the model function
+         and returns an ordered dict with the model names as the key and the
+         component results as the values.
 
    fit_report(modelpars=None, show_correl=True, min_correl=0.1)
          return a fit report.
 
-   plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--',
+   plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--', xlabel = None, ylabel=None,
             numpoints=None,  data_kws=None, fit_kws=None, init_kws=None,
             ax_kws=None)
         Plot the fit results using matplotlib.
@@ -670,7 +711,7 @@ class ModelFit(Minimizer):
                   ax_kws=None)
         Plot the fit residuals using matplotlib.
 
-   plot(self, datafmt='o', fitfmt='-', initfmt='--', numpoints=None,
+   plot(self, datafmt='o', fitfmt='-', initfmt='--', xlabel=None, ylabel=None, numpoints=None,
         data_kws=None, fit_kws=None, init_kws=None, ax_res_kws=None,
         ax_fit_kws=None, fig_kws=None)
         Plot the fit results and residuals using matplotlib.
@@ -682,6 +723,7 @@ class ModelFit(Minimizer):
         self.data = data
         self.weights = weights
         self.method = method
+        self.ci_out = None
         self.init_params = deepcopy(params)
         Minimizer.__init__(self, model._residual, params, fcn_args=fcn_args,
                            fcn_kws=fcn_kws, iter_cb=iter_cb,
@@ -692,42 +734,78 @@ class ModelFit(Minimizer):
         if data is not None:
             self.data = data
         if params is not None:
-            self.params = params
+            self.init_params = params
         if weights is not None:
             self.weights = weights
         if method is not None:
             self.method = method
+        self.ci_out = None
         self.userargs = (self.data, self.weights)
         self.userkws.update(kwargs)
-        self.init_params = deepcopy(self.params)
+        self.init_fit    = self.model.eval(params=self.params, **self.userkws)
+
+        _ret = self.minimize(method=self.method)
+
+        for attr in dir(_ret):
+            if not attr.startswith('_') :
+                try:
+                    setattr(self, attr, getattr(_ret, attr))
+                except AttributeError:
+                    pass
+
         self.init_values = self.model._make_all_args(self.init_params)
-        self.init_fit    = self.model.eval(params=self.init_params, **self.userkws)
+        self.best_values = self.model._make_all_args(_ret.params)
+        self.best_fit    = self.model.eval(params=_ret.params, **self.userkws)
 
-        self.minimize(method=self.method)
-        self.best_fit = self.model.eval(params=self.params, **self.userkws)
-        self.best_values = self.model._make_all_args(self.params)
+    def eval(self, params=None, **kwargs):
+        """
+        evaluate model function
+        Arguments:
+            params (Parameters):  parameters, defaults to ModelResult .params
+            kwargs (variable):  values of options, independent variables, etc
 
-    def eval(self, **kwargs):
+        Returns:
+            ndarray or float for evaluated model
+        """
         self.userkws.update(kwargs)
-        return self.model.eval(params=self.params, **self.userkws)
+        if params is None:
+           params = self.params
+        return self.model.eval(params=params, **self.userkws)
 
-    def eval_components(self, **kwargs):
+    def eval_components(self, params=None, **kwargs):
+        """
+        evaluate each component of a composite model function
+        Arguments:
+            params (Parameters):  parameters, defaults to ModelResult .params
+            kwargs (variable):  values of options, independent variables, etc
+
+        Returns:
+            ordered dictionary with keys of prefixes, and values of values for
+            each component of the model.
+        """
         self.userkws.update(kwargs)
-        return self.model.eval_components(params=self.params, **self.userkws)
+        if params is None:
+            params = self.params
+        return self.model.eval_components(params=params, **self.userkws)
 
-    def fit_report(self, modelpars=None, show_correl=True, min_correl=0.1):
+    def conf_interval(self, **kwargs):
+        """return explicitly calculated confidence intervals"""
+        if self.ci_out is None:
+            self.ci_out = conf_interval(self, self, **kwargs)
+        return self.ci_out
+
+    def ci_report(self, with_offset=True, ndigits=5, **kwargs):
+        """return nicely formatted report about confidence intervals"""
+        return ci_report(self.conf_interval(**kwargs),
+                         with_offset=with_offset, ndigits=ndigits)
+
+    def fit_report(self,  **kwargs):
         "return fit report"
-        stats_report = fit_report(self, modelpars=modelpars,
-                                 show_correl=show_correl,
-                                 min_correl=min_correl)
-        buff = ['[[Model]]']
-        buff.append('    %s' % self.model._reprstring(long=True))
-        buff = '\n'.join(buff)
-        out = '%s\n%s' % (buff, stats_report)
-        return out
+        return '[[Model]]\n    %s\n%s\n' % (self.model._reprstring(long=True),
+                                            fit_report(self, **kwargs))
 
     @_ensureMatplotlib
-    def plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--',
+    def plot_fit(self, ax=None, datafmt='o', fitfmt='-', initfmt='--', xlabel=None, ylabel=None, yerr=None,
                  numpoints=None,  data_kws=None, fit_kws=None, init_kws=None,
                  ax_kws=None):
         """Plot the fit results using matplotlib.
@@ -747,6 +825,12 @@ class ModelFit(Minimizer):
             matplotlib format string for fitted curve
         initfmt : string, optional
             matplotlib format string for initial conditions for the fit
+        xlabel : string, optional
+            matplotlib format string for labeling the x-axis
+        ylabel : string, optional
+            matplotlib format string for labeling the y-axis
+        yerr : ndarray, optional
+            array of uncertainties for data array
         numpoints : int, optional
             If provided, the final and initial fit curves are evaluated not
             only at data points, but refined to contain `numpoints` points in
@@ -770,15 +854,16 @@ class ModelFit(Minimizer):
         For details about plot format strings and keyword arguments see
         documentation of matplotlib.axes.Axes.plot.
 
-        If the fit model included weights, then matplotlib.axes.Axes.errorbar
-        is used to plot the data, with yerr set to 1 / self.weights**0.5
+        If yerr is specified or if the fit model included weights, then
+        matplotlib.axes.Axes.errorbar is used to plot the data.  If yerr is
+        not specified and the fit includes weights, yerr set to 1/self.weights
 
         If `ax` is None then matplotlib.pyplot.gca(**ax_kws) is called.
 
         See Also
         --------
-        ModelFit.plot_residuals : Plot the fit residuals using matplotlib.
-        ModelFit.plot : Plot the fit results and residuals using matplotlib.
+        ModelResult.plot_residuals : Plot the fit residuals using matplotlib.
+        ModelResult.plot : Plot the fit results and residuals using matplotlib.
         """
         if data_kws is None:
             data_kws = {}
@@ -803,32 +888,39 @@ class ModelFit(Minimizer):
 
         # make a dense array for x-axis if data is not dense
         if numpoints is not None and len(self.data) < numpoints:
-            x_array_dense = np.linspace(x_array[0], x_array[-1], numpoints)
+            x_array_dense = np.linspace(min(x_array), max(x_array), numpoints)
         else:
             x_array_dense = x_array
 
         ax.plot(x_array_dense, self.model.eval(self.init_params,
                 **{independent_var: x_array_dense}), initfmt,
-                label='initial for ' + self.model.name, **init_kws)
+                label='init', **init_kws)
         ax.plot(x_array_dense, self.model.eval(self.params,
                 **{independent_var: x_array_dense}), fitfmt,
-                label=self.model.name, **fit_kws)
+                label='best-fit', **fit_kws)
 
-        if self.weights is not None:
-            ax.errorbar(x_array, self.data, yerr=1/self.weights**0.5,
+        if yerr is None and self.weights is not None:
+            yerr = 1.0/self.weights
+        if yerr is not None:
+            ax.errorbar(x_array, self.data, yerr=yerr,
                         fmt=datafmt, label='data', **data_kws)
         else:
             ax.plot(x_array, self.data, datafmt, label='data', **data_kws)
 
-        ax.set_xlabel(independent_var)
-        ax.set_ylabel('y')
+        ax.set_title(self.model.name)
+        if xlabel is None:
+            ax.set_xlabel(independent_var)
+        else: ax.set_xlabel(xlabel)
+        if ylabel is None:
+            ax.set_ylabel('y')
+        else: ax.set_ylabel(ylabel)
         ax.legend()
 
         return ax
 
     @_ensureMatplotlib
-    def plot_residuals(self, ax=None, datafmt='o', data_kws=None, fit_kws=None,
-                       ax_kws=None):
+    def plot_residuals(self, ax=None, datafmt='o', yerr=None, data_kws=None,
+                       fit_kws=None, ax_kws=None):
         """Plot the fit residuals using matplotlib.
 
         The method will plot residuals of the fit using matplotlib, including:
@@ -842,6 +934,8 @@ class ModelFit(Minimizer):
             current pyplot axis or create one if there is none.
         datafmt : string, optional
             matplotlib format string for data points
+        yerr : ndarray, optional
+            array of uncertainties for data array
         data_kws : dictionary, optional
             keyword arguments passed on to the plot function for data points
         fit_kws : dictionary, optional
@@ -858,15 +952,16 @@ class ModelFit(Minimizer):
         For details about plot format strings and keyword arguments see
         documentation of matplotlib.axes.Axes.plot.
 
-        If the fit model included weights, then matplotlib.axes.Axes.errorbar
-        is used to plot the data, with yerr set to 1 / self.weights**0.5
+        If yerr is specified or if the fit model included weights, then
+        matplotlib.axes.Axes.errorbar is used to plot the data.  If yerr is
+        not specified and the fit includes weights, yerr set to 1/self.weights
 
         If `ax` is None then matplotlib.pyplot.gca(**ax_kws) is called.
 
         See Also
         --------
-        ModelFit.plot_fit : Plot the fit results using matplotlib.
-        ModelFit.plot : Plot the fit results and residuals using matplotlib.
+        ModelResult.plot_fit : Plot the fit results using matplotlib.
+        ModelResult.plot : Plot the fit results and residuals using matplotlib.
         """
         if data_kws is None:
             data_kws = {}
@@ -889,25 +984,28 @@ class ModelFit(Minimizer):
 
         x_array = self.userkws[independent_var]
 
-        ax.axhline(0, label=self.model.name, **fit_kws)
+        ax.axhline(0, **fit_kws)
 
-        if self.weights is not None:
-            ax.errorbar(x_array, self.eval() - self.data,
-                        yerr=1 / self.weights**0.5, fmt=datafmt,
-                        label='residuals', **data_kws)
+        if yerr is None and self.weights is not None:
+            yerr = 1.0/self.weights
+        if yerr is not None:
+            ax.errorbar(x_array, self.eval() - self.data, yerr=yerr,
+                        fmt=datafmt, label='residuals', **data_kws)
         else:
             ax.plot(x_array, self.eval() - self.data, datafmt,
                     label='residuals', **data_kws)
 
+        ax.set_title(self.model.name)
         ax.set_ylabel('residuals')
         ax.legend()
 
         return ax
 
     @_ensureMatplotlib
-    def plot(self, datafmt='o', fitfmt='-', initfmt='--', numpoints=None,
-             fig=None, data_kws=None, fit_kws=None, init_kws=None,
-             ax_res_kws=None, ax_fit_kws=None, fig_kws=None):
+    def plot(self, datafmt='o', fitfmt='-', initfmt='--', xlabel=None, ylabel=None, yerr=None,
+             numpoints=None, fig=None, data_kws=None, fit_kws=None,
+             init_kws=None, ax_res_kws=None, ax_fit_kws=None,
+             fig_kws=None):
         """Plot the fit results and residuals using matplotlib.
 
         The method will produce a matplotlib figure with both results of the
@@ -922,6 +1020,12 @@ class ModelFit(Minimizer):
             matplotlib format string for fitted curve
         initfmt : string, optional
             matplotlib format string for initial conditions for the fit
+        xlabel : string, optional
+            matplotlib format string for labeling the x-axis
+        ylabel : string, optional
+            matplotlib format string for labeling the y-axis
+        yerr : ndarray, optional
+            array of uncertainties for data array
         numpoints : int, optional
             If provided, the final and initial fit curves are evaluated not
             only at data points, but refined to contain `numpoints` points in
@@ -949,17 +1053,18 @@ class ModelFit(Minimizer):
 
         Notes
         ----
-        The method combines ModelFit.plot_fit and ModelFit.plot_residuals.
+        The method combines ModelResult.plot_fit and ModelResult.plot_residuals.
 
-        If the fit model included weights, then matplotlib.axes.Axes.errorbar
-        is used to plot the data, with yerr set to 1 / self.weights**0.5
+        If yerr is specified or if the fit model included weights, then
+        matplotlib.axes.Axes.errorbar is used to plot the data.  If yerr is
+        not specified and the fit includes weights, yerr set to 1/self.weights
 
         If `fig` is None then matplotlib.pyplot.figure(**fig_kws) is called.
 
         See Also
         --------
-        ModelFit.plot_fit : Plot the fit results using matplotlib.
-        ModelFit.plot_residuals : Plot the fit residuals using matplotlib.
+        ModelResult.plot_fit : Plot the fit results using matplotlib.
+        ModelResult.plot_residuals : Plot the fit residuals using matplotlib.
         """
         if data_kws is None:
             data_kws = {}
@@ -986,10 +1091,11 @@ class ModelFit(Minimizer):
         ax_res = fig.add_subplot(gs[0], **ax_res_kws)
         ax_fit = fig.add_subplot(gs[1], sharex=ax_res, **ax_fit_kws)
 
-        self.plot_fit(ax=ax_fit, datafmt=datafmt, fitfmt=fitfmt,
-                      initfmt=initfmt, numpoints=numpoints, data_kws=data_kws,
-                      fit_kws=fit_kws, init_kws={}, ax_kws=ax_fit_kws)
-        self.plot_residuals(ax=ax_res, datafmt=datafmt, data_kws=data_kws,
-                            fit_kws=fit_kws, ax_kws=ax_res_kws)
+        self.plot_fit(ax=ax_fit, datafmt=datafmt, fitfmt=fitfmt, yerr=yerr,
+                      initfmt=initfmt, xlabel=xlabel, ylabel=ylabel, numpoints=numpoints, data_kws=data_kws,
+                      fit_kws=fit_kws, init_kws=init_kws, ax_kws=ax_fit_kws)
+        self.plot_residuals(ax=ax_res, datafmt=datafmt, yerr=yerr,
+                            data_kws=data_kws, fit_kws=fit_kws,
+                            ax_kws=ax_res_kws)
 
         return fig
